VERSION 5.00
Begin VB.Form Form1 
   Caption         =   "3D Click Demo - Joshua Smyth (Peachysoft)"
   ClientHeight    =   7455
   ClientLeft      =   60
   ClientTop       =   450
   ClientWidth     =   9855
   KeyPreview      =   -1  'True
   LinkTopic       =   "Form1"
   ScaleHeight     =   7455
   ScaleWidth      =   9855
   StartUpPosition =   3  'Windows Default
   Begin VB.PictureBox viewport 
      Appearance      =   0  'Flat
      BackColor       =   &H80000005&
      ForeColor       =   &H00FFFFFF&
      Height          =   7215
      Left            =   120
      ScaleHeight     =   479
      ScaleMode       =   3  'Pixel
      ScaleWidth      =   639
      TabIndex        =   0
      Top             =   120
      Width           =   9615
   End
End
Attribute VB_Name = "Form1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'#################################################################
'#
'#  Title           :   3D Mouse Click
'#  Description     :   An example program on converting a 2D
'#                      mouse click and projecting it into
'#                      3D space.
'#  Author          :   Joshua Smyth
'#  Date Started    :   12 / 04 / 2004
'#
'#  Notes           :   This program is complement to a tutorial.
'#                      This code may be used in anyway for your
'#                      own purposes.
'#
'#  Additional Credit   - Jack Hoxley (www.directx4vb.com)
'#
'##################################################################
Option Explicit

'DirectX
Dim dx As DirectX8                  'DirectX itself
Dim D3D As Direct3D8                'Direct 3D
Dim D3DX As D3DX8                   'DirectX helper library, full of useful stuff
Dim D3DDevice As Direct3DDevice8    'The user's 3D card

Dim bRunning As Boolean             'Is the program running?

'Rectangle to be rendered
Dim vBuffer As Direct3DVertexBuffer8
Dim rect(5) As LITVERTEX


'Matricies
Dim matWorld As D3DMATRIX           'How the vertices are positioned
Dim matView As D3DMATRIX            'Where the camera is/where it's looking
Dim matProj As D3DMATRIX            'How the camera projects the 3D world onto the 2D screen

Const ASPECT = 1
Const SCREEN_WIDTH = 640
Const SCREEN_HEIGHT = 480

'Camera Variables
Dim CameraPos As D3DVECTOR
Dim CameraLookAt As D3DVECTOR
Dim CameraNearClip As Single
Dim CameraFarClip As Single

Private Sub Form_KeyDown(KeyCode As Integer, Shift As Integer)
    If KeyCode = vbKeyEscape Then bRunning = False
End Sub

Private Sub Form_Load()
    Me.Show
    bRunning = InitialiseDX
    
    Do While bRunning = True
        Call Render
        DoEvents
    Loop
    
    'End the program
    On Error Resume Next
        Set D3DDevice = Nothing
        Set D3D = Nothing
        Set dx = Nothing
        Debug.Print "All Objects Destroyed"

    Unload Me
End
End Sub

'#################################
'#                               #
'# Render                        #
'#                               #
'#################################

Private Sub Render()
    D3DDevice.Clear 0, ByVal 0, D3DCLEAR_TARGET Or D3DCLEAR_ZBUFFER, 0, 1#, 0  'Clear the screen black
    D3DDevice.BeginScene
    
        'Draw the rectangle
        D3DDevice.SetStreamSource 0, vBuffer, Len(rect(0))
        D3DDevice.DrawPrimitive D3DPT_TRIANGLELIST, 0, 1

    D3DDevice.EndScene
    D3DDevice.Present ByVal 0, ByVal 0, 0, ByVal 0
End Sub

'#################################
'#                               #
'# Initalisation Functions Below #
'#                               #
'#################################

Private Function InitialiseDX() As Boolean
On Error GoTo ErrHandler:

    Dim DispMode As D3DDISPLAYMODE          'Describes our Display Mode
    Dim D3DWindow As D3DPRESENT_PARAMETERS  'Describes our Viewport

    Set dx = New DirectX8                   'Create our Master DirectX Object
    Set D3D = dx.Direct3DCreate()           'Create Direct3D
    Set D3DX = New D3DX8                    'Create the D3DX helper library

    'Windowed mode
    D3D.GetAdapterDisplayMode D3DADAPTER_DEFAULT, DispMode  'Retrieve the current display mode
    D3DWindow.Windowed = 1                                  'Lets use Windowed Mode
    D3DWindow.BackBufferFormat = DispMode.Format            'Use current display mode format
    D3DWindow.EnableAutoDepthStencil = 1                    'Setup Depth Buffer
    D3DWindow.AutoDepthStencilFormat = D3DFMT_D16           '16 bit Z-Buffer
    D3DWindow.SwapEffect = D3DSWAPEFFECT_COPY_VSYNC

    'Create the 3D device
    Set D3DDevice = D3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, viewport.hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, D3DWindow)

    D3DDevice.SetVertexShader Lit_FVF                   'Set the vertex shader
    D3DDevice.SetRenderState D3DRS_LIGHTING, False      'Disable lighting
    D3DDevice.SetRenderState D3DRS_ZENABLE, 1           'enable Z buffer
    
    'Matricies
    D3DXMatrixIdentity matWorld
    D3DDevice.SetTransform D3DTS_WORLD, matWorld

    'Setup Camera
    CameraPos = MakeVector(0, 0, -12)
    CameraLookAt = MakeVector(0, 0, 0)
    CameraNearClip = 0.1
    CameraFarClip = 100

    
    D3DXMatrixLookAtLH matView, CameraPos, CameraLookAt, MakeVector(0, 1, 0)
    D3DDevice.SetTransform D3DTS_VIEW, matView

    D3DXMatrixPerspectiveFovLH matProj, FOV, ASPECT, CameraNearClip, CameraFarClip
    D3DDevice.SetTransform D3DTS_PROJECTION, matProj

    'Setup our geometry
    If InitialiseGeometry() = True Then
        InitialiseDX = True '//We succeeded
        Exit Function
    End If

'An Error occured
ErrHandler:
    MsgBox "Error Number Returned: " & Err.Number
    InitialiseDX = False
End Function

Private Function InitialiseGeometry()
On Error GoTo BailOut:

    rect(0) = CreateLitVertex(0, 3, 0, clWHITE, 0, 0, 0)
    rect(1) = CreateLitVertex(3, -1, 4, clSHADOW, 0, 0, 0)
    rect(2) = CreateLitVertex(-3, -1, 0, clWHITE, 0, 0, 0)

    'Put these verticies into the vertex buffer
    Set vBuffer = D3DDevice.CreateVertexBuffer(Len(rect(0)) * 6, 0, Lit_FVF, D3DPOOL_DEFAULT)
    If vBuffer Is Nothing Then Exit Function
    D3DVertexBuffer8SetData vBuffer, 0, Len(rect(0)) * 6, 0, rect(0)

    InitialiseGeometry = True
    Exit Function

'An Error occured
BailOut:
    InitialiseGeometry = False
End Function


Private Sub viewport_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    Dim Alpha As Single

    Dim MouseX As Single
    Dim MouseY As Single
    
    Dim vDir As D3DVECTOR
    Dim vIntersect As D3DVECTOR
    
    Dim halfwidth As Long
    Dim halfheight As Long
    
    halfwidth = SCREEN_WIDTH / 2
    halfheight = SCREEN_HEIGHT / 2
    
    Alpha = FOV / 2
    
    'Screen Width is 640x480
    MouseX = Tan(Alpha) * (X / halfwidth - 1) / ASPECT
    MouseY = Tan(Alpha) * (1 - Y / halfheight)
    
    'Intersect the plane from the eye to the centre of the plane

    Dim p1 As D3DVECTOR 'StartPoint on the nearplane
    Dim p2 As D3DVECTOR 'EndPoint on the farplane
    
    p1.X = MouseX * CameraNearClip
    p1.Y = MouseY * CameraNearClip
    p1.Z = CameraNearClip
    
    p2.X = MouseX * CameraFarClip
    p2.Y = MouseY * CameraFarClip
    p2.Z = CameraFarClip
    
    'Inverse the view matrix
    Dim matInverse As D3DMATRIX
    D3DDevice.GetTransform D3DTS_VIEW, matView
  
    D3DXMatrixInverse matInverse, 0, matView
    
    VectorMatrixMultiply p1, p1, matInverse
    VectorMatrixMultiply p2, p2, matInverse
    D3DXVec3Subtract vDir, p2, p1
    
    'Check if the points hit
    Dim v1 As D3DVECTOR
    Dim v2 As D3DVECTOR
    Dim v3 As D3DVECTOR
    
    Dim v4 As D3DVECTOR
    Dim v5 As D3DVECTOR
    Dim v6 As D3DVECTOR
    
    Dim pPlane1 As D3DVECTOR4

    
    v1 = MakeVector(rect(0).X, rect(0).Y, rect(0).Z)
    v2 = MakeVector(rect(1).X, rect(1).Y, rect(1).Z)
    v3 = MakeVector(rect(2).X, rect(2).Y, rect(2).Z)

    pPlane1 = Create4DPlaneVectorFromPoints(v1, v2, v3)
    
    Dim hit As Boolean
    hit = RayIntersectPlane(pPlane1, p1, vDir, vIntersect)
    If hit = True Then
        hit = PointInTriangle(v1, v2, v3, vIntersect)
    End If
    
    'Debug.Print hit
    MsgBox hit
End Sub
